﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;
using System.Windows.Input;
using System.Windows.Markup;

namespace CommandsSample
{
    [ContentProperty("Commands")]
    class CommandGroup : ICommand
    {
        public event EventHandler CanExecuteChanged;

        readonly ObservableCollection<ICommand> commands;

        public ObservableCollection<ICommand> Commands
        {
            get { return commands; }
        }

        public CommandGroup()
        {
            commands = new ObservableCollection<ICommand>();
            commands.CollectionChanged += OnCommandsChanged;
        }

        protected virtual void OnCanExecuteChanged(EventArgs e)
        {
            if (CanExecuteChanged != null)
                CanExecuteChanged(this, e);
        }

        private void OnChildCanExecuteChanged(object sender, EventArgs e)
        {
            OnCanExecuteChanged(e);
        }

        protected virtual void OnCommandsChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null && e.NewItems.Count > 0)
            {
                foreach (ICommand command in e.NewItems)
                    command.CanExecuteChanged += OnChildCanExecuteChanged;
            }

            if (e.OldItems != null && e.OldItems.Count > 0)
            {
                foreach (ICommand command in e.OldItems)
                    command.CanExecuteChanged -= OnChildCanExecuteChanged;
            }

            OnCanExecuteChanged(EventArgs.Empty);
        }

        public bool CanExecute(object parameter)
        {
            return commands.All(command => command.CanExecute(parameter));
        }

        public void Execute(object parameter)
        {
            foreach (ICommand command in commands)
                command.Execute(parameter);
        }
    }
}
