﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows.Markup

<ContentProperty("Commands")>
Class CommandGroup : Implements ICommand
    Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    ReadOnly _commands As ObservableCollection(Of ICommand)

    Public ReadOnly Property Commands As ObservableCollection(Of ICommand)
        Get
            Return _commands
        End Get
    End Property

    Public Sub New()
        _commands = New ObservableCollection(Of ICommand)()
        AddHandler _commands.CollectionChanged, AddressOf OnCommandsChanged
    End Sub

    Protected Overridable Sub OnCanExecuteChanged(e As EventArgs)
        RaiseEvent CanExecuteChanged(Me, e)
    End Sub

    Private Sub OnChildCanExecuteChanged(sender As Object, e As EventArgs)
        OnCanExecuteChanged(e)
    End Sub

    Protected Overridable Sub OnCommandsChanged(sender As Object, e As NotifyCollectionChangedEventArgs)
        If e.NewItems IsNot Nothing AndAlso e.NewItems.Count > 0 Then
            For Each command As ICommand In e.NewItems
                AddHandler command.CanExecuteChanged, AddressOf OnChildCanExecuteChanged
            Next
        End If

        If e.OldItems IsNot Nothing AndAlso e.OldItems.Count > 0 Then
            For Each command As ICommand In e.NewItems
                RemoveHandler command.CanExecuteChanged, AddressOf OnChildCanExecuteChanged
            Next
        End If

        OnCanExecuteChanged(EventArgs.Empty)
    End Sub

    Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
        Return _commands.All(Function(command) command.CanExecute(parameter))
    End Function

    Public Sub Execute(parameter As Object) Implements ICommand.Execute
        For Each command As ICommand In _commands
            command.Execute(parameter)
        Next
    End Sub
End Class
