﻿Class RelayCommand : Implements ICommand
    Private ReadOnly _methodToExecute As Action
    Private ReadOnly _canExecuteEvaluator As Func(Of Boolean)

    Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
        AddHandler(value As EventHandler)
            AddHandler CommandManager.RequerySuggested, value
        End AddHandler

        RemoveHandler(value As EventHandler)
            RemoveHandler CommandManager.RequerySuggested, value
        End RemoveHandler

        RaiseEvent(sender As Object, e As EventArgs)
        End RaiseEvent
    End Event

    Public Sub New(methodToExecute As Action, canExecuteEvaluator As Func(Of Boolean))
        _methodToExecute = methodToExecute
        _canExecuteEvaluator = canExecuteEvaluator
    End Sub

    Public Sub New(methodToExecute As Action)
        Me.New(methodToExecute, Function() True)
    End Sub

    Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
        Return _canExecuteEvaluator.Invoke()
    End Function

    Public Sub Execute(parameter As Object) Implements ICommand.Execute
        _methodToExecute.Invoke()
    End Sub
End Class
